'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var builders = require('../ast/builders.cjs');
var index = require('../fast-paths/index.cjs');
var guards = require('../guards.cjs');
var iterator = require('../iterator.cjs');
var emitCall = require('../templates/emit-call.cjs');
var scope = require('../templates/scope.cjs');
var tree = require('../tree/tree.cjs');
var generators = require('./generators.cjs');

function baseline(jsonPaths, opts) {
  const tree$1 = new tree.default(opts);
  const hashes = new Map();
  const callbacks = new Map();

  traverse: for (const [id, nodes] of jsonPaths) {
    const iterator$1 = new iterator.default(nodes);

    if (iterator$1.length === -1) {
      continue;
    }

    const hash = JSON.stringify(iterator$1.nodes);
    const existingHash = hashes.get(hash);

    if (existingHash !== void 0) {
      void (
        callbacks.get(existingHash)?.push(id) ??
        callbacks.set(existingHash, [id])
      );

      const method = tree$1.getMethodByHash(existingHash);
      let body = method.body.body;

      body.push(emitCall.default(id, iterator$1.modifiers));
      continue;
    } else {
      hashes.set(hash, id);
    }

    if (nodes.length > 0 && guards.isDeep(nodes[0])) {
      tree$1.traversalZones.destroy();
    }

    const ctx = {
      id,
      iterator: iterator$1,
    };

    tree$1.ctx = ctx;

    for (const fastPath of index.default) {
      if (fastPath(nodes, tree$1, ctx)) {
        continue traverse;
      }
    }

    const branch =
      iterator$1.feedback.minimumDepth !== -1
        ? [
            builders.ifStatement(
              builders.binaryExpression(
                iterator$1.feedback.fixed && iterator$1.feedback.stateOffset === -1
                  ? '!=='
                  : '<',
                scope.default.depth,
                builders.numericLiteral(iterator$1.feedback.minimumDepth + 1),
              ),
              builders.returnStatement(),
            ),
          ]
        : [];

    const zone = tree$1.traversalZones.create();

    for (const node of iterator$1) {
      if (guards.isDeep(node)) {
        zone?.allIn();
      }

      switch (node.type) {
        case 'MemberExpression':
          generators.generateMemberExpression(branch, iterator$1, node);
          zone?.expand(node.value);
          break;
        case 'MultipleMemberExpression':
          generators.generateMultipleMemberExpression(branch, iterator$1, node);
          zone?.expandMultiple(node.value);
          break;
        case 'SliceExpression':
          generators.generateSliceExpression(branch, iterator$1, node, tree$1);
          zone?.resize();
          break;
        case 'ScriptFilterExpression':
          generators.generateFilterScriptExpression(branch, iterator$1, node, tree$1);
          zone?.resize();
          break;
        case 'WildcardExpression':
          generators.generateWildcardExpression(branch, iterator$1);
          zone?.resize();
          break;
      }
    }

    branch.push(emitCall.default(ctx.id, iterator$1.modifiers));

    if (iterator$1.feedback.stateOffset !== -1) {
      tree$1.push(builders.stringLiteral(id), 'stateful-traverse');
      tree$1.push(builders.blockStatement(branch), 'stateful-tree-method');
    } else {
      tree$1.push(builders.stringLiteral(id), 'traverse');
      tree$1.push(builders.blockStatement(branch), 'tree-method');
    }

    zone?.attach();
  }

  return tree$1;
}

exports.default = baseline;
