'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var jsep = require('../../parser/jsep.cjs');
var builders = require('../ast/builders.cjs');
var generators = require('../baseline/generators.cjs');
var guards = require('../guards.cjs');
var emitCall = require('../templates/emit-call.cjs');
var scope = require('../templates/scope.cjs');

// covers:
// $..[?(@.bar])
// $[?(@.bar])


const TOP_LEVEL_DEPTH_IF_STATEMENT = builders.ifStatement(
  builders.binaryExpression('!==', scope.default.depth, builders.numericLiteral(1)),
  builders.returnStatement(),
);

var onlyFilterScriptExpression = (nodes, tree, ctx) => {
  if (nodes.length !== 1 || !guards.isScriptFilterExpression(nodes[0])) {
    return false;
  }

  const condition = builders.unaryExpression(
    '!',
    generators.rewriteESTree(tree, jsep.default(nodes[0].value)),
    true,
  );

  tree.pushAll([
    [
      builders.blockStatement([
        ...(guards.isDeep(nodes[0]) ? [] : [TOP_LEVEL_DEPTH_IF_STATEMENT]),
        builders.ifStatement(condition, builders.returnStatement()),
        emitCall.default(ctx.id, ctx.iterator.modifiers),
      ]),
      'tree-method',
    ],
    [builders.stringLiteral(ctx.id), 'traverse'],
  ]);

  if (!guards.isDeep(nodes[0])) {
    tree.traversalZones.create()?.resize().attach();
  }

  return true;
};

exports.default = onlyFilterScriptExpression;
